/*
   Implementation of a community detection algorithm based on modularity maximization
   including leading eigenvalue bisections, Kernighan-Lin-like fine-tuning, global
   final-tuning, agglomeration tuning, and providing a z-score effect size based on
   an Erdős-Rényi null model. The code also includes two simple proofs of concept (POC).
   Both proofs of concept illustrate how to run the code on a network stored as adjacency
   matrix or adjacency list, with the second POC including an external call to the freely
   available Graphviz software (www.graphviz.org), which produces a visualization of
   the networks with the communities separated by distance and distinguished by colour,
   for increased ease of understanding, using a tuned force-driven spring model.

   Copyright 2012-2015 Charo I. Del Genio

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/








#define _ISOC99_SOURCE




/* Header files to include */
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<string.h>
#include<unistd.h>




/* Global variables and structure definitions*/
/* idum is an auxiliary integer variable */
/* *ivdum, *ivdum2, *ivdum3 and *ivdumS are auxiliary integer vectors */
/* toler is the calculation tolerance */
/* *dvdum and *dvdum2 are auxiliary double vectors */
typedef struct{
	double modularity;
	double zscore;
	int *community;
} comstruct;
static int gN, N, idum, compnum, *gadcl, *gadcs, *adcl, *adcs, *seq, *comsiz, *subcl, *subcs, *map, *invmap, *cmap, *cinvmap, *subseq, *s, *s2, *ivdum, *ivdum2, *ivdum3, *ivdumS, *block, *ds, *compsize, *marked, **comlist, **finlist;
static long int gtwom, twom, *ctwom;
static double toler, tolerpm, tolacc, *lead, *dvdum, *dvdum2, *diags, **finalchg;
static comstruct result, globresult;




/* Prototypes */
static void   quicksort(int *s, const int length);
static void   prepmem(void);
static void   compmat(int **adj);
static void   makesub(const int com);
static void   makecomp(const int component);
static double powmtd(const int size, double *leadv, double *dtg2m, double (*rng)(void));
static double KLin(double (*rng)(void));
static double final(int *comnum, double (*rng)(void));
static double agglom(int *comnum, double (*rng)(void));
void      modinitlist(int **list, const int *sequence, const int nodes);
void      modinitmat(int **adj, const int nodes);
void      modclean(void);
comstruct modularity(double (*rng)(void), const double tol, const double tolpm, const double taccpt);
