/*
   Implementation of a community detection algorithm based on modularity maximization
   including leading eigenvalue bisections, Kernighan-Lin-like fine-tuning, global
   final-tuning, agglomeration tuning, and providing a z-score effect size based on
   an Erdős-Rényi null model. The code also includes two simple proofs of concept (POC).
   Both proofs of concept illustrate how to run the code on a network stored as adjacency
   matrix or adjacency list, with the second POC including an external call to the freely
   available Graphviz software (www.graphviz.org), which produces a visualization of
   the networks with the communities separated by distance and distinguished by colour,
   for increased ease of understanding, using a tuned force-driven spring model.

   Copyright 2012-2015 Charo I. Del Genio

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/








#define _XOPEN_SOURCE




/* Structure definition */
typedef struct{
	double modularity;
	double zscore;
	int *community;
} comstruct;




/* Prototypes */
void      modinitlist(int **list, const int *sequence, const int nodes);
void      modinitmat(int **adj, const int nodes);
void      modclean(void);
comstruct modularity(double (*rng)(void), const double tol, const double tolpm, const double taccpt);
